﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Windows.Forms;

namespace SyumasGen
{
    /// <summary>
    /// フレーム画像生成クラス
    /// </summary>
    class ImageGen : IDisposable
    {
        private Bitmap _bmp = null;
        private Graphics _g = null;

        internal const int _BaseWidth = 512;
        internal const int _BaseHeight = 384;

        internal static readonly Color _BackColorYellow = Color.FromArgb(255, 255, 204);

        /// <summary>
        /// インスタンスを初期化します。
        /// </summary>
        public ImageGen()
        {
            // グラフィックス描画の準備
            _bmp = new Bitmap(_BaseWidth, _BaseHeight);
            _g = Graphics.FromImage(_bmp);

            // レンダリング品質にアンチエイリアスを指定
            _g.SmoothingMode = SmoothingMode.AntiAlias;
            _g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
        }

        /// <summary>
        /// 割り当てられているリソースを破棄します。
        /// </summary>
        public void Dispose()
        {
            try
            {
                _g.Dispose();
                _bmp.Dispose();
            }
            catch (Exception)
            { }
        }

        /// <summary>
        /// 画像生成メイン処理
        /// </summary>
        /// <param name="dataList"></param>
        /// <param name="settings"></param>
        internal void Genarate(DataList dataList, Settings settings)
        {
            // 画像生成先パス
            string framePath = Path.Combine(Application.StartupPath, "frame");

            // ランキングフレーム生成
            for (int i = 0; i < dataList.RankingList.Count; i++)
            {
                // フレームの生成
                ImageRankingFrame.DrawFrame(_g, dataList, settings, i);
                _bmp.Save(Path.Combine(framePath, String.Format("rankingFrame{0}.png", i + 1)), ImageFormat.Png);

                // "xx Rank Up!"等の画像の生成
                Size size = ImageRankingFrame.DrawRankInfo(_g, dataList, settings, i);
                Image saveImg = _bmp.Clone(new Rectangle(0, 0, size.Width, size.Height), _bmp.PixelFormat);
                saveImg.Save(Path.Combine(framePath, String.Format("rankingText{0}.png", i + 1)), ImageFormat.Png);
                saveImg.Dispose();
            }

            // ピックアップフレームの生成
            for (int i = 0; i < dataList.PickupList.Count; i++)
            {
                // フレームの生成
                ImagePickupFrame.DrawVideoInfo(_g, dataList, settings, i);
                _bmp.Save(Path.Combine(framePath, String.Format("pickupFrame{0}.png", i + 1)), ImageFormat.Png);

                // ピックアップ解説
                for (int j = 0; j < 3; j++)
                {
                    if (dataList.PickupList[i].GetRemarks(j) != "")
                    {
                        ImagePickupFrame.DrawVideoExplain(_g, dataList, settings, i, j);
                        _bmp.Save(Path.Combine(framePath, String.Format("pickupText{0}_{1}.png", new object[] { i + 1, j + 1 })), ImageFormat.Png);
                    }
                }
            }

            // 順位カットイン画像生成
            for (int i = 0; i < dataList.TakagiList.Count; i++)
            {
                ImageRankCutIn.Draw(_g, dataList, settings, i);
                _bmp.Save(Path.Combine(framePath, String.Format("cutinText{0}.png", i + 1)), ImageFormat.Png);
            }

            // 除外リスト画像生成
            int startIndex = 0;
            int page = 1;
            while (startIndex < dataList.JogaiList.Count)
            {
                startIndex = ImageJogai.Draw(_g, dataList, settings, startIndex);
                _bmp.Save(Path.Combine(framePath, String.Format("jogaiList{0}.png", page++)), ImageFormat.Png);
            }

            // 集計概要画像生成
            {
                ImageSummary.Draw(_g, dataList, settings);
                _bmp.Save(Path.Combine(framePath, "summary.png"), ImageFormat.Png);
            }

            // 登録数及び再生数の順位リスト画像生成
            {
                ImagePlayRank.Draw(_g, dataList, settings);
                _bmp.Save(Path.Combine(framePath, "playRank.png"), ImageFormat.Png);
            }

            // 圏外リスト画像生成
            startIndex = 1; // 圏外リストは1から（0はタイトル）
            page = 1;
            while (startIndex < dataList.OutRangeList.Count)
            {
                startIndex = ImageOutRangeList.Draw(_g, dataList, settings, startIndex);
                _bmp.Save(Path.Combine(framePath, String.Format("outRangeList{0}.png", page++)), ImageFormat.Png);
            }
        }

        /// <summary>
        /// 文字列を描画します。
        /// </summary>
        /// <param name="s"></param>
        /// <param name="font"></param>
        /// <param name="brush"></param>
        /// <param name="pos"></param>
        /// <param name="align"></param>
        internal static int DrawText(Graphics g, string s, Font font, Brush brush, Point pos, Align align)
        {
            if (String.IsNullOrEmpty(s)) return 0;

            int width = GetStringWidth(g, s, font, pos);
            pos.X = GetAlignX(pos.X, width, align);
            g.DrawString(s, font, brush, pos);

            return width;
        }

        internal static int DrawText(Graphics g, string s, Font font, Brush brush, int x, int y, Align align)
        {
            return DrawText(g, s, font, brush, new Point(x, y), align);
        }

        /// <summary>
        /// 縁取りされた文字列を描画します。
        /// </summary>
        /// <param name="g"></param>
        /// <param name="s"></param>
        /// <param name="font"></param>
        /// <param name="textColor"></param>
        /// <param name="haloColor"></param>
        /// <param name="thickness"></param>
        /// <param name="pos"></param>
        /// <param name="align"></param>
        internal static int DrawFuchidoriText(Graphics g, string s, Font font, Color textColor, Color haloColor, float thickness, Point pos, Align align)
        {
            if (String.IsNullOrEmpty(s)) return 0;

            int width = GetStringWidth(g, s, font, pos);
            pos.X = GetAlignX(pos.X, width, align);

            GraphicsPath path = new GraphicsPath();
            path.AddString(s, font.FontFamily, (int)font.Style, font.Size, pos, StringFormat.GenericDefault);
            g.FillPath(new SolidBrush(textColor), path);
            g.DrawPath(new Pen(new SolidBrush(haloColor), thickness), path);

            return width;
        }

        internal static int DrawFuchidoriText(Graphics g, string s, Font font, Color textColor, Color haloColor, float thickness, int x, int y, Align align)
        {
            return DrawFuchidoriText(g, s, font, textColor, haloColor, thickness, new Point(x, y), align);
        }

        /// <summary>
        /// 指定した文字列を指定したフォントで描画したときの横幅を取得します。
        /// </summary>
        /// <param name="g"></param>
        /// <param name="s"></param>
        /// <param name="font"></param>
        /// <param name="pos"></param>
        /// <returns></returns>
        internal static int GetStringWidth(Graphics g, string s, Font font, Point pos)
        {
            if (String.IsNullOrEmpty(s)) return 0;

            SizeF strSize = g.MeasureString(s, font);
            RectangleF strRect = new RectangleF(pos, strSize);
            CharacterRange[] cr = { new CharacterRange(0, s.Length) };

            StringFormat sf = new StringFormat();
            sf.SetMeasurableCharacterRanges(cr);

            Region[] r = g.MeasureCharacterRanges(s, font, strRect, sf);
            float width = r[0].GetBounds(g).Width;
            return (int)(width);
        }

        internal static int GetAlignX(int x, int width, Align align)
        {
            int ret = x;

            switch (align)
            {
                case Align.Left:
                    break;

                case Align.Center:
                    ret = (int)(x - width / 2);
                    break;

                case Align.ScreenCenter:
                    ret = (int)((_BaseWidth - width) / 2);
                    break;

                case Align.Right:
                    ret = x - width;
                    break;
            }

            return ret;
        }

        internal static Color GetColorName(string colorName, Color defaultColor)
        {
            return colorName != "" ? Color.FromName(colorName) : defaultColor;
        }
    }

    /// <summary>
    /// 文字描画位置揃え
    /// </summary>
    internal enum Align
    {
        Left,
        Center,
        ScreenCenter,
        Right,
    }
}
