﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Windows.Forms;
using System.Collections.Generic;

namespace SyumasGen
{
    /// <summary>
    /// ランキングのフレーム画像
    /// </summary>
    class ImageRankingFrame
    {
        /// <summary>
        /// ランキングフレーム画像を描画します。
        /// </summary>
        /// <param name="g"></param>
        /// <param name="data1"></param>
        /// <param name="option"></param>
        /// <param name="index"></param>
        internal static void DrawFrame(Graphics g, DataList dataList, Settings option, int index)
        {
            RankingData data = dataList.RankingList[index];

            // 描画エリアのクリア
            g.Clear(Color.FromArgb(0, Color.Black));

            // フレームYオフセット設定
            Size offset = new Size(0, data.FrameYOffset);

            // フレーム情報
            FrameInfoData frameData = option.FrameInfoList[data.Mark];

            // フレーム画像描画
            string framePath = Path.Combine(Application.StartupPath, frameData.FrameImageFile);
            g.DrawImage(Image.FromFile(framePath), option.FramePos + offset);

            // 「順位」設定取得と描画幅算出
            TextInfoData rankInfo = option.TextInfoList[TextInfoKind.Rank];
            Point rankPos = new Point(rankInfo.PosX, rankInfo.PosY);
            Font rankFont = new Font(rankInfo.FontName, rankInfo.FontSize, FontStyle.Bold | FontStyle.Italic, GraphicsUnit.Pixel);

            int rankWidth = (int)g.MeasureString(data.Rank.ToString(), rankFont).Width;

            // 「タイトル」設定取得と描画幅算出
            TextInfoData titleInfo = option.TextInfoList[TextInfoKind.Title];
            Font titleFont = new Font(titleInfo.FontName, titleInfo.FontSize, FontStyle.Bold);
            Point titlePos = new Point(rankInfo.PosX + rankWidth, titleInfo.PosY);

            int titleWidth = (int)g.MeasureString(data.Title, titleFont).Width;

            // 「タイトル」が画面端からはみ出しそうだったら、順位を上にずらし、タイトルを左端から
            Size rankOffset = new Size(0, 0);

            if (titlePos.X + titleWidth > ImageGen._BaseWidth - 10)
            {
                rankOffset.Height = -25;
                titlePos.X = 10;
            }

            // 「順位変動」描画
            TextInfoData rankStatInfo = option.TextInfoList[TextInfoKind.RankStat];
            Point rankStatPos = new Point(rankStatInfo.PosX, rankStatInfo.PosY) + rankOffset + offset;
            Font rankStatFont = new Font(rankStatInfo.FontName, rankStatInfo.FontSize, FontStyle.Bold, GraphicsUnit.Pixel);

            ImageGen.DrawFuchidoriText(g, data.RankStat, rankStatFont, frameData.RankColor, Color.White, 1.5f, rankStatPos, Align.Left);

            rankStatFont.Dispose();

            // 「順位」描画
            rankPos += rankOffset + offset;
            Point rankShadowPos = rankPos + new Size(3, 3);
            Color shadowColor = Color.FromArgb(128, Color.Black);

            ImageGen.DrawFuchidoriText(g, data.Rank.ToString(), rankFont, shadowColor, shadowColor, 2.0f, rankShadowPos, Align.Left);
            ImageGen.DrawFuchidoriText(g, data.Rank.ToString(), rankFont, frameData.RankColor, Color.White, 2.0f, rankPos, Align.Left);

            rankFont.Dispose();

            // 「タイトル」描画
            titlePos += offset;

            g.DrawString(data.Title, titleFont, Brushes.Black, titlePos);

            titleFont.Dispose();

            // ---------- 動画情報
            TextInfoData videoDataInfo = option.TextInfoList[TextInfoKind.VideoData];
            Font videoDataFont = new Font(videoDataInfo.FontName, videoDataInfo.FontSize, FontStyle.Bold);
            Point pos = new Point(5, videoDataInfo.PosY) + offset;

            pos.Y = 335;
            pos += offset;

            // ポイント
            Color ptsColor = ImageGen.GetColorName(data.PtsColor, Color.Black);

            pos.X += ImageGen.DrawText(g, data.Pts.ToString("#,##0"), videoDataFont, new SolidBrush(ptsColor), pos, Align.Left);
            pos.X += ImageGen.DrawText(g, "pts", videoDataFont, Brushes.Black, pos, Align.Left) + 10;

            // 登録
            Color mylistColor = ImageGen.GetColorName(data.MylistColor, Color.Black);

            pos.X += ImageGen.DrawText(g, "登録:", videoDataFont, Brushes.Black, pos, Align.Left);
            pos.X += ImageGen.DrawText(g, data.Mylist.ToString("#,##0"), videoDataFont, new SolidBrush(mylistColor), pos, Align.Left) + 10;

            // 再生
            Color playColor = ImageGen.GetColorName(data.PlayColor, Color.Black);

            pos.X += ImageGen.DrawText(g, "再生:", videoDataFont, Brushes.Black, pos, Align.Left);
            pos.X += ImageGen.DrawText(g, data.Play.ToString("#,##0"), videoDataFont, new SolidBrush(playColor), pos, Align.Left) + 10;

            // ---------- 動画情報下段
            pos.X = 295;
            pos.Y = 358;
            pos += offset;

            // 作者名
            ImageGen.DrawText(g, data.Author, videoDataFont, Brushes.Black, pos, Align.Right);

            // 動画ID
            pos.X = 385;
            ImageGen.DrawText(g, data.VideoId, videoDataFont, Brushes.Black, pos, Align.Right);

            // 投稿日
            pos.X = 500;
            ImageGen.DrawText(g, data.Date.ToString("[yyyy/MM/dd]"), videoDataFont, Brushes.Black, pos, Align.Right);


            //----
            videoDataFont.Dispose();

            // 補足
            if (data.Remarks != "")
            {
                TextInfoData remarksInfo = option.TextInfoList[TextInfoKind.Remarks];
                Font remarksFont = new Font(remarksInfo.FontName, remarksInfo.FontSize, FontStyle.Regular, GraphicsUnit.Pixel);
                pos.X = ImageGen._BaseWidth - 10;
                pos.Y = remarksInfo.PosY;
                int remarksWidth = ImageGen.GetStringWidth(g, data.Remarks, remarksFont, pos);
                pos.X -= remarksWidth;
                pos += offset;

                ImageGen.DrawFuchidoriText(g, data.Remarks, remarksFont, Color.Black, Color.White, 5.0f, pos, Align.Left);
                g.DrawString(data.Remarks, remarksFont, Brushes.Black, pos);

                remarksFont.Dispose();
            }


            //// 「ポイント」描画
            //TextInfoData ptsInfo = option.TextInfoList[TextInfoKind.VideoData];
            //Font ptsFont = new Font(ptsInfo.FontName, 20, FontStyle.Bold);
            //Color ptsColor = ImageGenerator.GetColorName(data.PtsColor, Color.Black);

            //ImageGenerator.DrawText(g, data.Pts.ToString("#,##0"), ptsFont, new SolidBrush(ptsColor), 100, 343, Align.Right);
            //ImageGenerator.DrawText(g, "pts.", ptsFont, Brushes.Black, 100, 343, Align.Left);

            //ptsFont.Dispose();

            //// 「登録」描画
            //TextInfoData mylistInfo = option.TextInfoList[TextInfoKind.VideoData];
            //Font mylistFont = new Font(mylistInfo.FontName, 14, FontStyle.Bold);
            //Color mylistColor=ImageGenerator.GetColorName(data.MylistColor,Color.Black);

            //ImageGenerator.DrawText(g, "登録:", mylistFont, Brushes.Black, 160, 338, Align.Left);
            //ImageGenerator.DrawText(g, data.Mylist.ToString("#,##0"), mylistFont, new SolidBrush(mylistColor), 275, 338,Align.Right);

            //mylistFont.Dispose();

            ////// 「再生」描画
            //TextInfoData playInfo = option.TextInfoList[TextInfoKind.VideoData];
            //Font playFont = new Font(playInfo.FontName, 14, FontStyle.Bold);
            //Color playColor = ImageGenerator.GetColorName(data.PlayColor, Color.Black);

            //ImageGenerator.DrawText(g, "再生:", playFont, Brushes.Black, 160, 358, Align.Left);
            //ImageGenerator.DrawText(g, data.Play.ToString("#,##0"), playFont, new SolidBrush(playColor), 275, 358, Align.Right);

            //playFont.Dispose();

            //// 作者名
            //TextInfoData authorInfo = option.TextInfoList[TextInfoKind.VideoData];
            //Font authorFont = new Font(authorInfo.FontName, 15, FontStyle.Bold);

            //ImageGenerator.DrawText(g, data.Author, authorFont, Brushes.Black, 500, 341, Align.Right);

            //authorFont.Dispose();

            //// 投稿日
            //TextInfoData dateInfo = option.TextInfoList[TextInfoKind.VideoData];
            //Font dateFont = new Font(dateInfo.FontName, 13, FontStyle.Bold);

            //ImageGenerator.DrawText(g, data.Date.ToString("[yyyy/MM/dd]"), dateFont, Brushes.Black, 500, 358,Align.Right);

            //dateFont.Dispose();

            //// 動画ID
            //TextInfoData videoIdInfo = option.TextInfoList[TextInfoKind.VideoData];
            //Font videoIdFont = new Font(videoIdInfo.FontName, 13, FontStyle.Bold);

            //ImageGenerator.DrawText(g, data.VideoId, videoIdFont, Brushes.Black, 380, 358, Align.Right);

            //videoIdFont.Dispose();

            ////----
            //videoDataFont.Dispose();

            //// 補足
            //if (data.Remarks != "")
            //{
            //    TextInfoData remarksInfo = option.TextInfoList[TextInfoKind.Remarks];
            //    Font remarksFont = new Font(remarksInfo.FontName, remarksInfo.FontSize, FontStyle.Regular, GraphicsUnit.Pixel);
            //    pos.X = ImageGenerator._BaseWidth - 10;
            //    pos.Y = remarksInfo.PosY;
            //    int remarksWidth = ImageGenerator.GetStringWidth(g, data.Remarks, remarksFont, pos);
            //    pos.X -= remarksWidth;
            //    pos += offset;

            //    ImageGenerator.DrawFuchidoriText(g, data.Remarks, remarksFont, Color.Black, Color.White, 5.0f, pos,Align.Left);
            //    g.DrawString(data.Remarks, remarksFont, Brushes.Black, pos);

            //    remarksFont.Dispose();
            //}
        }

        /// <summary>
        /// ランキング順位変動情報文字列画像を描画します。
        /// </summary>
        /// <param name="g"></param>
        /// <param name="data1"></param>
        /// <param name="option"></param>
        /// <param name="index"></param>
        internal static Size DrawRankInfo(Graphics g, DataList dataList, Settings option, int index)
        {
            RankingData data = dataList.RankingList[index];

            // 描画エリアのクリア
            g.Clear(Color.FromArgb(0, Color.Black));

            // 表示内容
            string text = data.DiffRank.ToString(option.FrameInfoList[data.Mark].Text);

            // フォントの指定
            TextInfoData info = option.TextInfoList[TextInfoKind.RankInfo];
            Font font = new Font(info.FontName, info.FontSize, FontStyle.Bold | FontStyle.Italic, GraphicsUnit.Pixel);

            // 文字列描画
            Color c = option.FrameInfoList[data.Mark].RankColor;
            ImageGen.DrawFuchidoriText(g, text, font, c, Color.White, 2.0f, 0, 0, Align.Left);

            // 画像サイズを返す
            Size size = g.MeasureString(text, font).ToSize();
            size.Width += 20;
            return size;
        }
    }
}
